/*
 * dmx512_tester.c
 *
 * Created: 12.08.2016 21:35:12
 * Author : karpe
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "segment.h"


#define USART_BAUDRATE 250000
#define UBRR_VALUE (F_CPU/16/USART_BAUDRATE-1)


#define COUNT_CHANNELS 6

unsigned char data[COUNT_CHANNELS];
unsigned int DMX_Current_Channel = 0;
unsigned int DMX_Start_Channel = 0;
char DMX_Break = 0;



void USART_Init()
{
	// Set baud rate
	UBRRH = (uint8_t)(UBRR_VALUE>>8);
	UBRRL = (uint8_t)UBRR_VALUE;
	/* Set frame format: 8data, 2stop bit */
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
	//enable reception
	UCSRB |= (1<<RXEN);
	//     
	UCSRB |= (1<<RXCIE);
}


void Port_Init()
{
	A_DDR |= (1<<A_PIN);
	B_DDR |= (1<<B_PIN);
	C_DDR |= (1<<C_PIN);
	D_DDR |= (1<<D_PIN);
	E_DDR |= (1<<E_PIN);
	F_DDR |= (1<<F_PIN);
	G_DDR |= (1<<G_PIN);
	D1_DDR |= (1<<D1_PIN);
	D2_DDR |= (1<<D2_PIN);
	D3_DDR |= (1<<D3_PIN);
}

void Set_Digit (int x) {
	RESET_SEGMENT(A_PORT, A_PIN);
	RESET_SEGMENT(B_PORT, B_PIN);
	RESET_SEGMENT(C_PORT, C_PIN);
	RESET_SEGMENT(D_PORT, D_PIN);
	RESET_SEGMENT(E_PORT, E_PIN);
	RESET_SEGMENT(F_PORT, F_PIN);
	RESET_SEGMENT(G_PORT, G_PIN);
	
	switch (x % 10)
	{
		case 0:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(E_PORT, E_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			break;
		case 1:
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			break;
		case 2:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(E_PORT, E_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 3:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 4:
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 5:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 6:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(E_PORT, E_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 7:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			break;
		case 8:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(E_PORT, E_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
		case 9:
			SET_SEGMENT(A_PORT, A_PIN);
			SET_SEGMENT(B_PORT, B_PIN);
			SET_SEGMENT(C_PORT, C_PIN);
			SET_SEGMENT(D_PORT, D_PIN);
			SET_SEGMENT(F_PORT, F_PIN);
			SET_SEGMENT(G_PORT, G_PIN);
			break;
	}
}

ISR(USART_RXC_vect)
{
	unsigned char UART_Status = UCSRA;
	unsigned char UART_Data = UDR;
	//   (Frame error)
	if (UART_Status & (1<<FE)){
		DMX_Current_Channel = 0;
		DMX_Break = 1;
	} else if (DMX_Break == 1){
		int index = DMX_Current_Channel - DMX_Start_Channel - 1;
		if ((index >= 0) && (index < COUNT_CHANNELS)){
			data[index] = UART_Data;
		}
		DMX_Current_Channel++;
	}
}

int main(void)
{
	USART_Init();
	Port_Init();
	
	for (int i = 0; i < COUNT_CHANNELS; i++) data[i] = 0;
	
	sei();
    /* Replace with your application code */
    while (1) 
    {
		SET_DIGIT_1();
		Set_Digit(data[5] / 100);
		_delay_ms(5);
		RESET_DIGIT_1();
		SET_DIGIT_2();
		Set_Digit((data[5] % 100) / 10);
		_delay_ms(5);
		RESET_DIGIT_2();
		SET_DIGIT_3();
		Set_Digit(data[5] % 10);
		_delay_ms(5);
		RESET_DIGIT_3();
    }
}

